package net.chlup.myshare;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.nio.charset.Charset;
import java.security.KeyManagementException;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.UnrecoverableKeyException;
import java.security.cert.CertificateException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.TimeZone;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.conn.scheme.Scheme;
import org.apache.http.conn.ssl.SSLSocketFactory;
import org.apache.http.entity.mime.MultipartEntity;
import org.apache.http.entity.mime.content.InputStreamBody;
import org.apache.http.entity.mime.content.StringBody;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.impl.client.DefaultHttpRequestRetryHandler;
import org.apache.http.util.EntityUtils;

import android.app.IntentService;
import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.ContentResolver;
import android.content.Context;
import android.content.Intent;
import android.database.Cursor;
import android.net.Uri;
import android.os.Bundle;
import android.os.Environment;
import android.os.PowerManager;
import android.provider.MediaStore;
import android.util.Base64;
import android.util.Log;

public class myshareservice extends IntentService {

   /* Prefs */
   Preferences prefs;
   DefaultHttpClient httpclient = null;
   URL target_url;

   /* Storage PATH */
   String storagepath = Environment.getExternalStorageDirectory().toString();

   /* Notification */
   NotificationManager notman;
   Notification notif;
   PendingIntent notintent;

   public myshareservice() {
      super("myshareservice");
   }

   @Override
   protected void onHandleIntent(Intent intent) {

      /* Get prefs */
      this.prefs = new Preferences(this);
      this.prefs.load();

      setupHttpConnection();

      /* Set URL */
      this.target_url = null;
      try {
         this.target_url = new URL(this.prefs.target_url);
      }
      catch (MalformedURLException e) {
         Log.e(this.getClass().getName(), e.toString());
         this.target_url = null;
      }

      /* nedovol, aby usnulo CPU behem  uploadu (pokud to sem nedam a usnuti dovolim, fungovalo to pekne, ale pri usnuti telefonu se upload prerusil a bezel az po jeho probuzeni) */
      PowerManager pm = (PowerManager) getSystemService(Context.POWER_SERVICE);
      PowerManager.WakeLock wl = pm.newWakeLock(PowerManager.PARTIAL_WAKE_LOCK, "myshareserviceStart - CPU no sleep");
      wl.acquire();
         setupNotif();
         myshareserviceStart(intent);
      wl.release();

   }

   private void setupHttpConnection() {
      this.httpclient = new DefaultHttpClient();
      this.httpclient.setHttpRequestRetryHandler(new DefaultHttpRequestRetryHandler(1, false)); // retry jen 1x (napriklad je nedostupna sit) (pokud bych to neuvedl, tak defaultne je tusim retry 3x)
      if(this.prefs.own_key_store) {
         try {
            KeyStore trustStore = KeyStore.getInstance(KeyStore.getDefaultType());
            FileInputStream instream = new FileInputStream(new File(storagepath + "/security/cacerts.bks"));
            trustStore.load(instream, "changeit".toCharArray());
            instream.close();
            SSLSocketFactory socketFactory = new SSLSocketFactory(trustStore);
            Scheme sch = new Scheme("https", socketFactory, 443); // deprecated
            //Scheme sch = new Scheme("https", 443, socketFactory); // takhle to ma byt nove, ale hrouti se to (prej: java.lang.NoSuchMethodError) - jako by to davalo prednost verzi co je v Androidu a ne te co jsem tam vsunul pres externi knihovny - tak zustavam pri stare verzi 
            this.httpclient.getConnectionManager().getSchemeRegistry().register(sch);
         }
         catch (KeyStoreException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
            httpclient = null;
         }
         catch (FileNotFoundException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
            httpclient = null;
         }
         catch (NoSuchAlgorithmException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
            httpclient = null;
         }
         catch (CertificateException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
            httpclient = null;
         }
         catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
            httpclient = null;
         }
         catch (KeyManagementException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
            httpclient = null;
         }
         catch (UnrecoverableKeyException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
            httpclient = null;
         }
      }
   }

   private void setupNotif() {
      /* Show state in notif bar */

      if (this.notman != null) {
         /* Remove notification */
         this.notman.cancelAll();
         this.notman = null;
      }

      /* Set up a persistent notification */
      this.notman = (NotificationManager)   getSystemService(Context.NOTIFICATION_SERVICE);
      this.notman.cancelAll();
      this.notif = new Notification(
            R.drawable.notif_icon,
            null, // neuvadim zadny text, aby zbytecne na status baru nebyly sachry
            System.currentTimeMillis()
      );
      //this.notif.flags = Notification.FLAG_ONGOING_EVENT;
      this.notif.flags = Notification.FLAG_AUTO_CANCEL;
      this.notintent = PendingIntent.getActivity(
            this,
            0,
            new Intent(this, myshare.class),
            0
      );
      this.notif.setLatestEventInfo(
            this,
            getString(R.string.app_name),
            myshareservice.this.getString(R.string.msgUploadStart),
            notintent
      );
      this.notman.notify(0, this.notif);
   }

   private void myshareserviceStart(Intent intent) {

      Bundle extras = intent.getExtras();
      String action = intent.getAction();

      if(this.target_url != null) {
         // if this is from the share menu
         if (Intent.ACTION_SEND.equals(action)) {
            if (extras.containsKey(Intent.EXTRA_STREAM)) {
               Log.i(this.getClass().getName(), "upload one image");
               // Get resource path from intent callee
               Uri uri = (Uri) extras.getParcelable(Intent.EXTRA_STREAM);
               String result = mainSendRequest(uri, extras.getString("MYSHARE_TYPETAG"));
               setNotif(0, 0, result);
               return;
            }
         }
         // if this is from the share menu
         else if (Intent.ACTION_SEND_MULTIPLE.equals(action)) {
            if (extras.containsKey(Intent.EXTRA_STREAM)) {
               Log.i(this.getClass().getName(), "upload list images");
               ArrayList<Uri> theFileList = intent.getParcelableArrayListExtra(Intent.EXTRA_STREAM);
               int id = 0;
               for(Uri uri: theFileList) {
                  String result = mainSendRequest(uri, extras.getString("MYSHARE_TYPETAG"));
                  setNotif(id, theFileList.size()-1, result);
                  ++id;
               }
               return;
            }
         }
      }

   }

   private String mainSendRequest(Uri uri, String typetag) {
      String result = "";
      // Query gallery for camera picture via
      // Android ContentResolver interface
      ContentResolver cr = getContentResolver();
      InputStream is;
      try {
         // zjisteni InputStream obrazku dle uri 
         is = cr.openInputStream(uri);
         // zjisteni, kdy byl soubor naposledy modifikovan ("vytvoren")
         String filenameshort = "unknown";
         String createdate = "";
         String corectiontogmt = "";
         String filename = getRealPathFromContentUri(uri);
         //Log.i(this.getClass().getName(), "uri: " + uri);
         //Log.i(this.getClass().getName(), "filename: " + filename);
         if(filename != null) {
            File file = new File(filename);
            if(file.exists()) {
               filenameshort = file.getName();
               long epochlastmodified=file.lastModified();
               //createdate = new java.text.SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new java.util.Date (epochlastmodified));
               SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
               sdf.setTimeZone(TimeZone.getTimeZone("UTC"));
               createdate = sdf.format(new Date(epochlastmodified));
               corectiontogmt = Integer.toString(0); // cas posilam v GMT a proto je korekce na GMT vzdy 0
               Log.i(this.getClass().getName(), "createdate: " + createdate);
            }
         }
         // samotne odeslani dat
         result = SendRequest(is, filenameshort, createdate, corectiontogmt, typetag);
      }
      catch (FileNotFoundException e) {
         result = myshareservice.this.getString(R.string.msgUploadFailed) + " (err 1)";
         Log.e(this.getClass().getName(), e.toString());
      }
      // Get binary bytes for encode
      return result;
   }

   private String SendRequest(InputStream is, String filenameshort, String createdate, String corectiontogmt, String typetag) {

      String result = "";

      /* reseni problemu s nepodarenou inicializaci httpclient
       *  - vetsinou pujde o chybu, ze nejsou dobre nacteny vlastni CA
       *    z lokalniho souboru */
      if(httpclient == null) {
         Log.e(this.getClass().getName(), "httpclient is null (may be problems with CA)");
         result = "Error: httpclient is null (may be problems with CA)";
         return result;
      }

      HttpPost httppost = new HttpPost(target_url.toString());
      if (this.prefs.http_user != null && this.prefs.http_password != null) {
         String http_userpassword = this.prefs.http_user + ":" + this.prefs.http_password;
         httppost.setHeader("Authorization", "basic " + Base64.encodeToString(http_userpassword.getBytes(),Base64.NO_WRAP));
      }

      HttpResponse response;

      try {
         String serverinfo="";

         MultipartEntity mpEntity = new MultipartEntity();
         mpEntity.addPart("img", new InputStreamBody(is, filenameshort));
         mpEntity.addPart("createdate", new StringBody(createdate));
         mpEntity.addPart("corectiontogmt", new StringBody(corectiontogmt));
         if(typetag.equals("tag")) {
            if (this.prefs.tag1 != null) {
               mpEntity.addPart("tag", new StringBody(this.prefs.tag1, Charset.forName("UTF-8")));
            }
            if (this.prefs.tag2 != null) {
               mpEntity.addPart("tag", new StringBody(this.prefs.tag2, Charset.forName("UTF-8")));
            }
         }

         httppost.setEntity(mpEntity);
         response = httpclient.execute(httppost);
         HttpEntity entity = response.getEntity();
         if (entity != null) {
            serverinfo=EntityUtils.toString(entity); // tohle muze asi pripadne nahradit radek nize (pokud budes chti s obsahem co je vracen pracovat)
            if(serverinfo.length()>100) {
               serverinfo=serverinfo.substring(0,100);
            }
            //entity.consumeContent(); // zajisti asi konzumaci obsahu s ukoncenim pozadavku - diky tomu funguje keep-alive (kdyz to tu nebylo, keep-alive nefungoval - nestacilo jen posunout definic httpclient nahoru)
         }
         result = myshareservice.this.getString(R.string.msgUploadedSuccessfully) +
                  " (SERVER: "+serverinfo+")";
      }
      catch (ClientProtocolException e) {
         Log.e(this.getClass().getName(), "ClientProtocolException: " + e.toString());
         result = "Error - ClientProtocolException";
      }
      catch (IOException e) {
         Log.e(this.getClass().getName(), "IOException: " + e.toString());
         result = myshareservice.this.getString(R.string.msgUploadFailed) + " (err 2)";
      }
      return result;
   }

   private void setNotif(int id, int maxid, String result) {
      if(maxid!=0) {
         result= Integer.toString(id+1) + "/" + Integer.toString(maxid+1) + " " + result;
      }
      this.notif.setLatestEventInfo(
            this,
            getString(R.string.app_name),
            result,
            notintent
      );
      this.notman.notify(id, this.notif);
   }

   // And to convert the image URI to the direct file system path of the image file
   public String getRealPathFromContentUri(Uri contentUri) {
      String filePath = null;
      if(contentUri.getScheme().compareTo("content")==0) {
         String [] filePathColumn = { MediaStore.Images.Media.DATA };
         Cursor cursor = getContentResolver().query(contentUri, filePathColumn, null, null, null);
         if(cursor != null) {
            cursor.moveToFirst();
            int columnIndex = cursor.getColumnIndex(filePathColumn[0]);
            filePath = cursor.getString(columnIndex);
            cursor.close();
         }
         //int column_index = cursor.getColumnIndexOrThrow(MediaStore.Images.Media.DATA);
      }
      else if(contentUri.getScheme().compareTo("file")==0) {
         filePath = contentUri.getPath();
      }
      return filePath;
   }

}
